//+-------------------------------------------------------------------------------------+
//|                                                                  SimpleReversal.mq5 |
//|                                                     Copyright 2022, MetaQuotes Ltd. |
//|                                                                https://www.mql5.com |
//+-------------------------------------------------------------------------------------+
#property copyright "Copyright 2022, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property description "The idea is to trade on reversal only."
#property description "Trader from Desa Satryan, Kanigoro, Blitar, East Java, Indonesia."
//---
#include<Trade\Trade.mqh>
#include<Trade\SymbolInfo.mqh>
#include<Trade\PositionInfo.mqh>
#include<Trade\AccountInfo.mqh>
//---
CTrade         m_trade;
CSymbolInfo    m_symbol;
CPositionInfo  m_position;
CAccountInfo   m_account;
//--- input parameters
input    string   Symbol="current";
input    double   LotSize=0.01;
input    double   SL=2.0;
input    double   TP=4.0;
static   int      handle_iFractals;
static   int      handle_iATR;
ENUM_ACCOUNT_MARGIN_MODE  m_margin_mode;
ulong    magic_number=123456789;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   SetMarginMode();
   if(!IsHedging())
     {
      Print("Hedging only!");
      return(INIT_FAILED);
     }
//---
   m_symbol.Name(Symbol());                  // sets symbol name
   if(!RefreshRates())
     {
      Print("Error RefreshRates. Bid=",DoubleToString(m_symbol.Bid(),Digits()),
            ", Ask=",DoubleToString(m_symbol.Ask(),Digits()));
      return(INIT_FAILED);
     }
   m_symbol.Refresh();
//---
   m_trade.SetExpertMagicNumber(magic_number);    // sets magic number
//--- create handle of the indicator 
   handle_iFractals=iFractals(m_symbol.Name(),Period());
//--- if the handle is not created 
   if(handle_iFractals==INVALID_HANDLE)
     {
      //--- tell about the failure and output the error code 
      PrintFormat("Failed to create handle of the indicator for the symbol %s/%s, error code %d",
                  m_symbol.Name(),
                  EnumToString(Period()),
                  GetLastError());
      //--- the indicator is stopped early 
      return(INIT_FAILED);
     }
//--- create handle of the indicator 
   handle_iATR=iATR(m_symbol.Name(),Period(),10);
//--- if the handle is not created 
   if(handle_iATR==INVALID_HANDLE)
     {
      //--- tell about the failure and output the error code 
      PrintFormat("Failed to create handle of the indicator for the symbol %s/%s, error code %d",
                  m_symbol.Name(),
                  EnumToString(Period()),
                  GetLastError());
      //--- the indicator is stopped early 
      return(INIT_FAILED);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   double currenthigh=iHigh(_Symbol,_Period,0);
   double currentlow=iLow(_Symbol,_Period,0);
   double highest=iHighest(_Symbol,_Period,MODE_HIGH,WHOLE_ARRAY,0);
   double lowest=iLowest(_Symbol,_Period,MODE_LOW,WHOLE_ARRAY,0);
   double fractals=iFractalsGet(MAIN_LINE,0);
   double atr=iATRGet(MAIN_LINE,0);
//---
   if(highest==fractals && highest>currenthigh && PositionsTotal()<=1)
      {
      m_trade.Sell(LotSize,_Symbol,m_symbol.Bid(),m_symbol.Bid()+SL*atr,m_symbol.Bid()-TP*atr,NULL);
      }
//---
   if(lowest==fractals && lowest<currentlow && PositionsTotal()<=1)
      {
      m_trade.Buy(LotSize,_Symbol,m_symbol.Ask(),m_symbol.Ask()-SL*atr,m_symbol.Ask()+TP*atr,NULL);
      }  
  }
//+------------------------------------------------------------------+
void SetMarginMode(void)
  {
   m_margin_mode=(ENUM_ACCOUNT_MARGIN_MODE)AccountInfoInteger(ACCOUNT_MARGIN_MODE);
  }
//---
bool IsHedging(void)
  {
   return(m_margin_mode==ACCOUNT_MARGIN_MODE_RETAIL_HEDGING);
  }
//+--------------------------------------------------------------------------------+
//| Refreshes the symbol quotes data and protects against the return value of zero |                               |
//+--------------------------------------------------------------------------------+
bool RefreshRates()
  {
//--- refresh rates
   if(!m_symbol.RefreshRates())
      return(false);
//--- protection against the return value of "zero"
   if(m_symbol.Ask()==0 || m_symbol.Bid()==0)
      return(false);
//---
   return(true);
  }
//---
double iFractalsGet(const int buffer,const int index)
  {
   double IND_FRACTALS[1];
//--- reset error code 
   ResetLastError();
//--- fill a part of the Buffer array with values from the indicator buffer that has 0 index 
   if(CopyBuffer(handle_iFractals,buffer,index,1,IND_FRACTALS)<0)
     {
      //--- if the copying fails, tell the error code 
      PrintFormat("Failed to copy data from the indicator, error code %d",GetLastError());
      //--- quit with zero result - it means that the indicator is considered as not calculated 
      return(0.0);
     }
   return(IND_FRACTALS[0]);
  }
//---
double iATRGet(const int buffer,const int index)
  {
   double IND_ATR[1];
//--- reset error code 
   ResetLastError();
//--- fill a part of the Buffer array with values from the indicator buffer that has 0 index 
   if(CopyBuffer(handle_iFractals,buffer,index,1,IND_ATR)<0)
     {
      //--- if the copying fails, tell the error code 
      PrintFormat("Failed to copy data from the indicator, error code %d",GetLastError());
      //--- quit with zero result - it means that the indicator is considered as not calculated 
      return(0.0);
     }
   return(IND_ATR[0]);
  }
//---